#!/usr/bin/perl

package RPM::Source::BundleImport;

use strict;
use warnings;

our $VERSION=0.021;
#use Cwd;
use Carp;
use File::Basename;
use RPM::Source::Transform 0.016;
use RPM::Source::Transformation::Component;
use RPM::Source::Transformation::GenericTransformationSet;
use RPM::Source::Transformation::Factory::RaiseRelease;
use RPM::Source::Tools::SourceArchive;
#use RPM::Source::Tools::BuildReqStorage;
use RPM::Source::Transformation::RPMEditorProvider;

our @ISA=qw/RPM::Source::Transformation::Component/;
unshift @RPM::Source::TransformContainer::SUPPORT_CLASSES, __PACKAGE__;
# TODO: move to TransformContainer? or drop ver and made get_spec_parent just an inherent method?
$RPM::Source::Transform::GET_SPEC_PARENT=__PACKAGE__;

$RPM::Source::Transformation::PkgWriter::outputdir='OUT';
$RPM::Source::Transformation::Factory::RaiseRelease::nextrel='incr';
$RPM::Source::Transformation::Factory::RaiseRelease::defaultchangelog='- updated by package builder';

my $new_package_changelog='- initial import by package builder';
my $regenerated_package_changelog='- regenerated from template by package builder';

our $builder_name='generic';
our $opt_template;
our $guess_pkg_name_version=\&_SourceBundleImport::helper::default_guess_pkg_name_version;
our $version_comparator;
our $opt_die_on_depth=0;
# due to rpm4.0
our $opt_force_ascii=1;
our $verbose=0;

my $opt_regenerate;
# TODO:
my $opt_specname;

our @LONGOPT=(
    'regenerate' => \$opt_regenerate,
    'force-ascii!' => \$opt_force_ascii,
    );
sub longopt_usage {
    print "RPM::Source::BundleImport specific options:\n";
    print "	--regenerate -- regenerate spec file from template.\n";
}

sub register_messages {
    my ($self,$extopt)=@_;
    ### TODO: use some fixed keys hash here
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-template => $opt_template ? $opt_template:
	 ($builder_name ? "templates/${builder_name}-package.spec" : 'templates/package.spec'),
	-specname => $opt_specname,
	-macros_pkg_name => ${RPM::Source::BundleImport::builder_name}.'_pkg_name',
	-macros_pkg_version => ${RPM::Source::BundleImport::builder_name}.'_pkg_version',
	-macros_meta_name => ${RPM::Source::BundleImport::builder_name}.'_meta_name',
	-macros_meta_version => ${RPM::Source::BundleImport::builder_name}.'_meta_version',
	-opt_bundle_name => undef,
	-opt_bundle_version => undef,
	-opt_bundle_url => undef,
	-opt_regenerate => $opt_regenerate,
	});
}

sub get_spec_parent {
    my ($self,$converter,$pkg,$converter_opt,$parentref)=@_;
    my $file=$pkg->file();
    ($converter_opt->{-opt_bundle_name},$converter_opt->{-opt_bundle_version})=&$guess_pkg_name_version(basename($file));
    $converter_opt->{-opt_bundle_url}=$pkg->url;
    $converter_opt->{-opt_copy_to_sources}||=[];
    push @{$converter_opt->{-opt_copy_to_sources}}, $file;
    my $specname=$converter_opt->{-specname};
    my @SPECARGS=(
	SPECFILE=>$converter_opt->{-template},
	-create_sourcedir=>'tmp',
	-specname=> $specname ? $specname : ($builder_name ? "${builder_name}-$converter_opt->{-opt_bundle_name}.spec":"$converter_opt->{-opt_bundle_name}.spec"),
	);

    if ($parentref) {
	$converter_opt->{-default_changelog} = $regenerated_package_changelog;
    } else {
	$converter_opt->{-default_changelog} = $new_package_changelog;
    }

    if ($parentref and not $converter_opt->{-opt_regenerate}) {
	$converter_opt->{-no_default_hook}=1;
	@SPECARGS=@$parentref;
    }

    my $parent=$RPM::Source::TransformContainer::RPMEditorProvider->rpmeditoropts2parent($parentref);
    my $spec=$RPM::Source::TransformContainer::RPMEditorProvider->rpmeditoropts2spec(\@SPECARGS);


    if ($parentref and $converter_opt->{-opt_regenerate}) {
	my $mainsec=$spec->main_section;
	my $parentmainsec=$parent->main_section;
	foreach my $tag (qw/Serial Epoch Version Release/) {
	    my $val=$parentmainsec->get_tag($tag);
	    $mainsec->set_tag($tag,$val) if defined $val;
	}
    }

    $self->update_spec_from_bundle($spec,$parent,$converter,$converter_opt,$file);
    return ($spec,$parent);
}

# that's INPUT_TRANSFORMER part
sub update_spec_from_bundle {
    my ($self,$spec,$parent,$converter_opt,$file)=@_;
}

package _SourceBundleImport::helper;

use strict;
use warnings;
use File::Basename;

my %LICENSE_MD5=qw/
500acbfd1f82ab7b2475fd776f02c093  Apache-1.0
92afa66b413bb09a3bbd590091a836f5  Apache-1.1
3b83ef96387f14655fc854ddc3c6bd57  Apache-2.0
f921793d03cc6d63ec4b15e9be8fd3f8  Artistic-1
ed0f22fa6f2ee0a77567130e0e8c8d15  Artistic-2
3775480a712fc46a69647678acb234cb  BSD
470546c892fdcedecf2800c71374165b  Cryptix
1c20993619c939907d3ccad7bf32b466  CyrusIMAP
7e6c3ccfe6dcf65b285400a8ab52ee1a  FDL-1.1
ad1419ecc56e060eccf8184a87c4285f  FDL-1.2
fc6dce656150c601a734bcc41dd47bed  GPL-1
892f569a555ba9c07a568a7c0c4fa63a  GPL-2
7783169b4be06b54e86730eb01bc3a31  GPL-2
4c61b8950dc1aab4d2aa7c2ae6b1cfb3  GPL-3
d32239bcb673463ab874e80d47fae504  GPL-3
30c0149b971538b028bf5bf9dfb46d3d  IPL-1.0
c9a74d60dc7eb386e7054e065c8c8fe3  ISC
b29c0845cfd8eb68f25dd58c624c161f  LGPL-2
243b725d71bb5df4a1e5920b344b86ad  LGPL-2.1
b52f2d57d10c4f7ee67a7eb9615d5d24  LGPL-3
243f3ec4753b6bc1d47e0c3c510645cc  MITX
197bc65329cc6b3ded0a5c3898ebd060  MPL-1.0
bfe1f75d606912a4111c90743d6c7325  MPL-1.1
ee04927ad5dd1e14f97a48c05bdb08b3  NOSL-1.0
a86020e24936cdb318147ff6c792a271  NPL-1.0
90ca7253bbdadd72e74e3ad045ef8875  NPL-1.1
8c4f6273162efcb83280d19c36c39bd1  OCL-1.0
9444e9ea223d1493101f89aca0b8c6de  OpenLDAP-2.3
88ae063404a460d693584298756a3baf  OpenPub-0.4
0e1e6aec764a1fdc679826495c9f8dd7  QPL-1.0
94851ea2e2f53419ac704d0cdabebed4  Sendmail
1247aba766f172392f4e7c44ec755c7f  Sleepycat
b0d85a62427ff2c2aad900a6a4784a27  SPL-1.0
129becfe0e62f242a447d8c6da697a86  W3C
864da87825d026354b6ca3b7459db3d0  XConsortium
fed5484a32507f6d545fed8ee6bb815f  XFree86-other
397b30d2cdd7b4757adeae9709f0e91b  Zlib
3e6271a943b4525617f1a2ec3b8207e1  ZPL-1.0
ed01b49aaeb35cd3f5951266b647cdb8  ZPL-2.1
    /;

my @KNOWN_LICENSE=qw/
Apache
Apache-1.0
Apache-1.1
Apache-2.0
Artistic
Artistic-1
Artistic-2
BSD
Cryptix
CyrusIMAP
FDL
FDL-1.1
FDL-1.2
GPL
GPL1
GPL2
GPL3
GPL-1
GPL-2
GPL-3
GPLv1
GPLv2
GPLv3
IPL-1.0
ISC
LGPL
LGPL2
LGPL21
LGPL3
LGPL-2
LGPL-2.1
LGPL-3
LGPLv2
LGPLv2.1
LGPLv3
MIT
MITX
MPL
MPL-1.0
MPL-1.1
NOSL-1.0
NPL-1.0
NPL-1.1
OCL-1.0
OpenLDAP-2.3
OpenPub-0.4
QPL-1.0
SPL-1.0
W3C
XConsortium
ZPL-1.0
ZPL-2.1
/;
my @KNOWN_COPYING=qw/
COPYING
LICENSE
/;
my %KNOWN_LICENSE=map {$_=>1} @KNOWN_LICENSE;
my %KNOWN_COPYING=map {$_=>1} @KNOWN_COPYING;

# the more fit first
my @KNOWN_README=qw/
README
README.txt
DESCR
description
READ.ME
README.md
README.mkd
README.mkdn
/;
our %KNOWN_DOCS=map {$_=>1} 
@KNOWN_COPYING, @KNOWN_LICENSE, @KNOWN_README, qw/
CommitLog
COMPAT
DEBUGGING debug.txt
DEPRECATED
deprecated.txt
design_doc.txt
design.txt
developer-guide
DEVELOPERS
Developers-Guide.sxw
DEVELOPERS.txt
development.txt
Develop_Readme
DISCLAIMER
DISCLAIMER.txt
History.md
KNOWNBUGS
KNOWN.BUGS
KNOWN_ISSUES
known-problems
KNOWN-PROBLEMS
MANUAL
Manual
manual
ReleaseNotes
RELEASENOTES
RELEASE_NOTES
RELEASE-NOTES
RELEASE.NOTES
ReleaseNotes.1
RELEASE_NOTES.HTML
RELEASES
RELEASE_TESTS
RELNOTES
tutorial
TUTORIAL
USAGE
WHATIS
WHATIS.jp
WHATSNEW
WARNING
/;
#AUTHORS README.md CHANGELOG.md

sub analyze_tarball_update_prep_source {
    my ($spec,$parent,$converter_opt,$file,$tarball)=@_;
    my @depth=$tarball->depth();
    die "$file: archive depth is ".(scalar @depth).". expected 0 or 1. unsupported layout.\n" if $opt_die_on_depth and @depth>1;

    # STUB; -T is do not unpack source0. possible future use is for specific types of archives.
    my $setupopt='-T -c %name-%version';
    my ($macros_pkg_name,$macros_pkg_version) =($converter_opt->{-macros_pkg_name},$converter_opt->{-macros_pkg_version});
    my ($tarname,$tarversion) =($converter_opt->{-opt_bundle_name},$converter_opt->{-opt_bundle_version});
    if (@depth==0) {
	$setupopt='-c %name-%version';
    } elsif (@depth==1) {
	$setupopt='-n '.&__rpm404_horrible_quote($depth[0]);
	$setupopt =~ s/\Q$tarname\E/\%\{$macros_pkg_name\}/ if defined $tarname;
	$setupopt =~ s/\Q$tarversion\E/\%\{$macros_pkg_version\}/ if defined $tarversion;
    } elsif (@depth>1) {
	$setupopt='-n '.&__rpm404_horrible_quote($depth[0]);
	$setupopt =~ s/\Q$tarname\E/\%\{$macros_pkg_name\}/ if defined $tarname;
	$setupopt =~ s/\Q$tarversion\E/\%\{$macros_pkg_version\}/ if defined $tarversion;
	my $depthpath=join('/',@depth[1..$#depth]);
	$spec->get_section('prep')->push_body(q{# depth is > 1 .. moving up
pushd "}.$depthpath.q{"
  mv -f * }.('../' x $#depth).q{
  mv -f .* }.('../' x $#depth).q{ ||:
popd}."\n");
    }
    $spec->get_section('prep')->exclude_body(qr'\%setup');
    $spec->get_section('prep')->unshift_body("\%setup -q $setupopt\n");
    my $source0=basename($file);
    my $sourceurl=$converter_opt->{-opt_bundle_url};
    $source0=$sourceurl if $sourceurl;
    $source0 =~ s/\Q$tarname\E/\%\{$macros_pkg_name\}/ if defined $tarname;
    $source0 =~ s/\Q$tarversion\E/\%\{$macros_pkg_version\}/ if defined $tarversion;
    $spec->main_section->set_tag('Source0',$source0);
    $spec->main_section->unshift_body('BuildRequires: unzip'."\n") if $file=~/\.(zip|jar)$/;
}

sub __rpm404_horrible_quote {
    my $in=shift;
    $in=~s,',\\'\\\\\\'\\',g; # ' -> \'\\\'\'
    return $in;
}

sub rebase_filelist {
    my ($tarball)=@_;
    my @depth=$tarball->depth();
    my $filelist=$tarball->filelist();
    return $filelist if not @depth;
    my $depth_prefix=join('/',@depth);
    my @rebased_list;
    foreach my $file (@$filelist) {
	my $rebased_file=$file;
	$rebased_file=~s,^\Q$depth_prefix\E/,,;
	push @rebased_list, $rebased_file;
    }
    return \@rebased_list;
}

sub rebased_filelist_to_depth1dir {
    my ($filelist)=@_;
    my %depth1dir;
    foreach my $file (@$filelist) {
	$depth1dir{$1}=1 if $file=~m!^([^/]+)/!;
    }
    return \%depth1dir;
}

sub guess_license {
    my ($unpack_path,$filelist)=@_;
    my $fallback;
    foreach my $copying (@KNOWN_COPYING) {
	my $file="$unpack_path".$copying;
	if (-s $file) {
	    $fallback='see '.$copying;
	    my ($md5sum)=split(/\s+/,`md5sum "$file"`);
	    print STDERR "$file: md5sum=$md5sum\n";
	    my $license=$LICENSE_MD5{$md5sum} if $LICENSE_MD5{$md5sum};
	    return $license if $license;
	}
    }
    my @found_licenses=grep {$KNOWN_LICENSE{$_}} @$filelist;
    return join(', ',@found_licenses) if @found_licenses;
    return $fallback;
}

sub guess_summary_description {
    my ($metadata,$unpack_path,$filelist)=@_;
    return if $metadata->{'description'} and $metadata->{'abstract'};
    &__update_metadata_abstract($metadata,guess_summary_from_pc_in($unpack_path,$filelist));
    return if $metadata->{'description'} and $metadata->{'abstract'};
    my ($summary,$description);
    foreach my $readme (@KNOWN_README) {
	my $readme_f="$unpack_path/$readme";
	if (-s $readme_f) {
	    ($summary,$description)=&guess_summary_description_from_README($readme_f);
	    &__update_metadata_description($metadata,$description);
	    &__update_metadata_abstract($metadata,$summary);
	    return if $metadata->{'description'} and $metadata->{'abstract'};
	}
    }
    # Further try
    #return if $metadata->{'description'} and $metadata->{'abstract'};
}

sub __update_metadata_description {
    my ($metadata,$description)=@_;
    $metadata->{'description'}||=$description;
}

sub __update_metadata_abstract {
    my ($metadata,$summary)=@_;
    if ($summary) {
	$summary=~s/^\s*//;
	$summary=~s/\s*$//;
    }
    $metadata->{'abstract'}||=$summary;
}

my $max_description=12;
sub guess_summary_description_from_README {
    my ($readme)=@_;
    my $description='';
    my $count=0;
    my ($presummary,$summary);
    #print STDERR "guess_summary_description_from_README: trying $readme\n";
    open(F, $readme) || die "guess_summary_description_from_README: $! [$readme].";
    local $_;
    my @README=<F>;
    close(F);
    my $section_type;
    my %section;
    foreach (@README) {
	if (/^(?:=head1|#)?\s*(NAMES|NAME|DESCRIPTION)\b/) {
	    $section_type=$1;
	    $section{$section_type}=[];
	} else {
	    push @{$section{$section_type}}, $_ if $section_type;
	}
    }
    my @names;
    push @names, @{$section{'NAME'}} if $section{'NAME'};
    push @names, @{$section{'NAMES'}} if $section{'NAMES'};
    ($summary)=grep {/\S/} @names;
    if ($section{'DESCRIPTION'}) {
	$#{$section{'DESCRIPTION'}}=$max_description if $#{$section{'DESCRIPTION'}}>$max_description;
	return ($summary,$description) if $summary;
    }

    foreach (@README) {
	last if $count> $max_description;
	#print STDERR "guess_s_d_R: $_";
	if (not $summary) {
	    if (/^\s*======================/ or /^\s*--------------/) {
		$summary=$presummary;
	    } elsif (/^Copyright/ and $count==1 and $presummary=~/\s-\s/) {
		$summary=$presummary;
	    }
	}
	$presummary=$_;
	$description.=$_;
	$count++;
    }
    #print STDERR "guess_summary_description_from_README: ",($summary ? $summary: '(none)'),"\n";
    return ($summary,$description);
}


sub guess_summary_from_pc_in {
    my ($unpack_path,$filelist)=@_;
    my $summary;
    my @candidates=grep {/\.pc\.in$/} @$filelist;
    return unless @candidates==1;
    open(F, "$unpack_path/$candidates[0]") || die "guess_summary_from_pc_in: $! [$candidates[0]].";
    local $_;
    while (<F>) {
	if (/^Description:\s*(.*)$/) {
	    $summary=$1;
	}
    }
    close(F);
    return $summary;
}

sub analyze_rebased_filelist_update_doc {
    my ($filessec,$filelist,$depth1dir)=@_;
    # todo: html, css
    # do we need it? install
    my %already_added;
    my @docs;
    foreach my $file (@$filelist) {
	# toplevel dir
	if ($file=~m!^(?:read[-_]?me|author|bugs|change_?log|changes|copying|copyright|todo|license)[^/]*$!i or $file=~m!^(?:read[-_]?me|sample|example|\.txt|\.pdf)$!i) {
	    &__add_if_not_added(\@docs,\%already_added, $file, $file);
	    next;
	};
	my $basename=basename($file);
	if ($KNOWN_DOCS{$basename}) {
	    &__add_if_not_added(\@docs,\%already_added, $basename, $file);
	    next;
	}
    }
    foreach my $docdir (qw/doc docs example examples manual Doc Docs Example Examples Manual MANUAL/) {
	push @docs, $docdir if $depth1dir->{$docdir};
    }

    $filessec->exclude_body(qr'^\%doc\s+');
    $filessec->unshift_body('%doc '.join(' ',@docs)."\n") if @docs;
}

sub __add_if_not_added {
    my ($docs,$already_added, $basename, $file)=@_;
    # cp: will not overwrite just-created ...LICENSE with LICENSE
    my $added=$already_added->{$basename};
    if (not $added) {
	push @$docs, $file;
    } elsif ($added ne $basename) {
	warn "%doc: multiple files $basename: $file $added - selected $file\n" if $verbose>1;
	# replace the one not from the top dir
	for (my $i=0; $i<@$docs; $i++) {
	    if ($docs->[$i] eq $added) {
		$docs->[$i]=$file;
		last;
	    }
	}
    } else {
	warn "%doc: multiple files $basename: $file $added - skipped $file\n" if $verbose>1;
    }
    $already_added->{$basename}=$file;
}

sub update_url {
    my ($section,$urlstring)=@_;
    if ($urlstring) {
	my ($url)=split(/[\s,]+/,$urlstring);
	$url=~s/%/%%/g;
	$section->set_tag('URL', $url) if $url;
    }
}

sub update_summary_description {
    my ($spec,$parent,$converter_opt,$meta_summary,$meta_description)=@_;
    if (not $parent or $converter_opt->{-opt_regenerate}) {
	if ($meta_summary) {
	    ($meta_summary)=split(/\n/,$meta_summary) if $meta_summary=~/\n/s;
	    $meta_summary=substr($meta_summary,0,99).'...' if length($meta_summary)>99;
	    $meta_summary=~s/%/% /g;
	    $meta_summary=&__kill_non_ascii($meta_summary);
	    if ($meta_summary!~/[A-Za-z]/m) {
		# warn here ...
	    } else {
		$spec->main_section->set_tag('Summary', $meta_summary);
	    }
	}
	if ($meta_description) {
	    $meta_description=~s/%/%%/g;
	    $meta_description=&__kill_non_ascii($meta_description);
	    if ($meta_description!~/^\s*[A-Za-z]/m) {
		# something like '# TODO' is ignored by rpm 4.0.4
		# warn here ...
	    } else {
		my $descrsection=$spec->get_section('description');
		$descrsection->empty_body();
		$descrsection->push_body($meta_description."\n");
	    }
	}
    }
}

sub redefine_macro {
    my ($section,$macros_name,$macros_value) =@_;
    Carp::confess "Oops: empty macros_name" unless $macros_name;
    $section->exclude_body(qr'^\%define\s+'.$macros_name);
    $section->unshift_body("\%define $macros_name $macros_value\n");
}

sub update_defines_pkg_meta_name {
    my ($mainsec,$converter_opt,$metadata_name,$tarname) =@_;
    my ($macros_pkg_name,$macros_meta_name) =($converter_opt->{-macros_pkg_name},$converter_opt->{-macros_meta_name});
    &redefine_macro($mainsec,$macros_meta_name,$metadata_name) if ($macros_meta_name and ($macros_meta_name ne 'name'));
    &redefine_macro($mainsec,$macros_pkg_name,$tarname) if ($macros_pkg_name and ($macros_pkg_name ne 'name'));
}

sub update_defines_pkg_meta_version {
    my ($mainsec,$converter_opt,$metadata_version,$tarversion) =@_;
    my $rpm_version=$tarversion;
    # nodejs release handler
    #print STDERR "1:rpm_version=$rpm_version\n";
    $rpm_version=~s/-\D+$//;
    #print STDERR "2:rpm_version=$rpm_version\n";
    $rpm_version=~s/(-\d+)\D+$/$1/;
    #print STDERR "3:rpm_version=$rpm_version\n";
    $rpm_version=~s/-/_/g;
    # version is set by Transform
    $converter_opt->{-version}=$rpm_version;
    #$mainsec->set_tag('Version', 0);
    # octave/nodejs specific? looks deprecated -drop it
    #my $_pkg_version=$tarversion;
    #$_pkg_version .= '.0' if $_pkg_version =~ /^\d+\.\d+$/;
    my ($macros_pkg_version,$macros_meta_version) =($converter_opt->{-macros_pkg_version},$converter_opt->{-macros_meta_version});
    &redefine_macro($mainsec,$macros_meta_version,$metadata_version) if ($macros_meta_version and $macros_meta_version ne 'version');
    &redefine_macro($mainsec,$macros_pkg_version,$tarversion) if ($macros_pkg_version and $macros_pkg_version ne 'version');
}

# TODO: trimmed version interface
sub check_pkg_meta_name_version {
    my ($mainsec,$converter_opt,$metadata_name,$metadata_version,$file) =@_;
    my $tarname=$converter_opt->{-opt_bundle_name};
    my $tarversion=$converter_opt->{-opt_bundle_version};
    my $expanded_metadata_version=$metadata_version;
    if ($metadata_version ne $tarversion and $metadata_version.'.0' eq $tarversion) {
	$expanded_metadata_version.='.0';
    }
    # hack around release in tarball (nodejs)
    # Oops: athet-0.0.1-alpha-8.tgz: file name athet-0.0.1-alpha and Name: athet tag does not match :(
    if (($metadata_name ne $tarname or $expanded_metadata_version ne $tarversion) and "$metadata_name-$expanded_metadata_version" eq "$tarname-$tarversion") {
	$tarname=$metadata_name;
	$tarversion=$metadata_version;
	$converter_opt->{-opt_bundle_name}=$tarname;
	$converter_opt->{-opt_bundle_version}=$tarversion;
    }
    if ($metadata_name ne $tarname) {
	my $trimmed_tarname=$tarname;
	$trimmed_tarname =~ s/'//;
	warn "check_pkg_meta: $file: archive file name $tarname and metadata name: $metadata_name does not match :( \n" if $trimmed_tarname ne $tarname;
	return 0;
    } elsif ($expanded_metadata_version ne $tarversion) {
	my $trimmed_tarversion=$tarversion;
	$trimmed_tarversion =~ s/-[A-Za-z]+$//;
	if ($RPM::Source::BundleImport::version_comparator) {
	    my $compare_result;
	    eval {
		$compare_result = &$RPM::Source::BundleImport::version_comparator($metadata_version,$tarversion)==0 ||
		    &$RPM::Source::BundleImport::version_comparator($metadata_version,$trimmed_tarversion)==0;
	    };
	    warn "check_pkg_meta: with version_comparator: $file: version comparator failed.\n" if $@;
	    return 0;
	    if ($compare_result) {
		return 1;
	    } else {
		warn "check_pkg_meta: with version_comparator: $file: archive file version $tarversion and metadata version: $metadata_version does not match :( \n";
		return 0;
	    }
	} elsif ($metadata_version eq $trimmed_tarversion) {
	    return 1;
	}
	warn "check_pkg_meta: $file: archive file version $tarversion and metadata version: $metadata_version does not match :( \n";
	return 0;
    }
}

sub check_update_pkg_meta_name_version {
    my ($mainsec,$converter_opt,$metadata_name,$metadata_version,$file) =@_;
    &check_pkg_meta_name_version($mainsec,$converter_opt,$metadata_name,$metadata_version,$file)
	or die "Name and version as listed in metadata does not match package's.\n";
    my $tarname=$converter_opt->{-opt_bundle_name};
    my $tarversion=$converter_opt->{-opt_bundle_version};
    &update_defines_pkg_meta_version($mainsec,$converter_opt,$metadata_version,$tarversion);
    &update_defines_pkg_meta_name($mainsec,$converter_opt,$metadata_name,$tarname);
}

sub default_guess_pkg_name_version {
    my ($file)=@_;
    die "Oops: default_guess_pkg_name_version: strange file $file" unless $file=~/^(.+)-v?([\d\.]+(?:-[A-Za-z]\w*|[abpn]\d+)?)\.(tar\..+|t[gbx]z2?|7z|tar|jar|zip)$/;
    my $bundle_name=$1;
    my $bundle_version=$2;
    print STDERR "default_guess_pkg_name_version: bundle_name=$bundle_name bundle_version=$bundle_version\n" if $verbose;
    return ($bundle_name,$bundle_version);
}

#use POSIX qw(locale_h);
sub __kill_non_ascii {
    my ($line)=@_;
    return $line if not $RPM::Source::BundleImport::opt_force_ascii;
    local $_=$line;
    # some distro use iso or utf8 in description. ALT Linux use ascii only. :(
    #tr/é/e/;
    tr/¡© éöóêåíãøùçõúôûâàïðîëäæýÿñìèòüáþÉÖÓÊÅÍÃØÙÇÕÚÔÛÂÀÏÐÎËÄÆÝßÑÌÈÒÜÁÞ\x98\x99/-.....eooeaiaoucououaaidieaayvnieouabEOOEAIAOUCOUOUAAIDIEAAYVNIEOUAB''''/;
    # exterminatus of the rest of the non-ascii characters
    s/[^[:ascii:]]/./g;  # get rid of non-ASCII characters
    s/[\x00-\x09\x0c\x0e-\x1f]/_/sg;  # get rid of control characters (ESC, ...)
#    s/[\x80-\xff]/./sg;
    # query and save the old locale
#    my $old_locale = setlocale(LC_CTYPE);
#    setlocale(LC_CTYPE, 'C');
#    s/[^[:print:]]/./;
#    # restore the old locale
#    setlocale(LC_CTYPE, $old_locale);
    return $_;
}


1;
